@@ -4,7 +4,7 @@
  * This file is part of mpop, a POP3 client.
  *
  * Copyright (C) 2000, 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010, 2011,
- * 2014, 2015, 2016, 2018, 2019
+ * 2014, 2015, 2016, 2018, 2019, 2020
  * Martin Lambers <marlam@marlam.de>
  * Martin Stenberg <martin@gnutiken.se> (passwordeval support)
  *
@@ -90,9 +90,11 @@ account_t *account_new(const char *conffile, const char *id)
     a->tls_nocertcheck = 0;
     a->tls_min_dh_prime_bits = -1;
     a->tls_priorities = NULL;
+    a->tls_host_override = NULL;
     a->proxy_host = NULL;
     a->proxy_port = 0;
     a->source_ip = NULL;
+    a->socketname = NULL;
     return a;
 }
 
@@ -172,9 +174,12 @@ account_t *account_copy(account_t *acc)
         a->tls_min_dh_prime_bits = acc->tls_min_dh_prime_bits;
         a->tls_priorities =
             acc->tls_priorities ? xstrdup(acc->tls_priorities) : NULL;
+        a->tls_host_override =
+            acc->tls_host_override ? xstrdup(acc->tls_host_override) : NULL;
         a->proxy_host = acc->proxy_host ? xstrdup(acc->proxy_host) : NULL;
         a->proxy_port = acc->proxy_port;
         a->source_ip = acc->source_ip ? xstrdup(acc->source_ip) : NULL;
+        a->socketname = acc->socketname ? xstrdup(acc->socketname) : NULL;
     }
     return a;
 }
@@ -210,8 +215,10 @@ void account_free(void *a)
         free(p->tls_sha1_fingerprint);
         free(p->tls_md5_fingerprint);
         free(p->tls_priorities);
+        free(p->tls_host_override);
         free(p->proxy_host);
         free(p->source_ip);
+        free(p->socketname);
         free(p);
     }
 }
@@ -576,6 +583,12 @@ void override_account(account_t *acc1, account_t *acc2)
         acc1->tls_priorities = acc2->tls_priorities
             ? xstrdup(acc2->tls_priorities) : NULL;
     }
+    if (acc2->mask & ACC_TLS_HOST_OVERRIDE)
+    {
+        free(acc1->tls_host_override);
+        acc1->tls_host_override = acc2->tls_host_override
+            ? xstrdup(acc2->tls_host_override) : NULL;
+    }
     if (acc2->mask & ACC_PROXY_HOST)
     {
         free(acc1->proxy_host);
@@ -590,6 +603,11 @@ void override_account(account_t *acc1, account_t *acc2)
         free(acc1->source_ip);
         acc1->source_ip = acc2->source_ip ? xstrdup(acc2->source_ip) : NULL;
     }
+    if (acc2->mask & ACC_SOCKET)
+    {
+        free(acc1->socketname);
+        acc1->socketname = acc2->socketname ? xstrdup(acc2->socketname) : NULL;
+    }
     acc1->mask |= acc2->mask;
 }
 
@@ -602,7 +620,12 @@ void override_account(account_t *acc1, account_t *acc2)
 
 int check_account(account_t *acc, int retrmail, char **errstr)
 {
-    if (!acc->host)
+    if (!acc->host && !acc->socketname)
+    {
+        *errstr = xasprintf(_("host not set"));
+        return CONF_ESYNTAX;
+    }
+    if (acc->tls && !(acc->host || acc->tls_host_override))
     {
         *errstr = xasprintf(_("host not set"));
         return CONF_ESYNTAX;
@@ -1409,6 +1432,19 @@ int read_conffile(const char *conffile, FILE *f, list_t **acc_list,
                 acc->tls_priorities = xstrdup(arg);
             }
         }
+        else if (strcmp(cmd, "tls_host_override") == 0)
+        {
+            acc->mask |= ACC_TLS_HOST_OVERRIDE;
+            free(acc->tls_host_override);
+            if (*arg == '\0')
+            {
+                acc->tls_host_override = NULL;
+            }
+            else
+            {
+                acc->tls_host_override = xstrdup(arg);
+            }
+        }
         else if (strcmp(cmd, "only_new") == 0)
         {
             acc->mask |= ACC_ONLY_NEW;
@@ -1558,6 +1594,19 @@ int read_conffile(const char *conffile, FILE *f, list_t **acc_list,
                 acc->source_ip = xstrdup(arg);
             }
         }
+        else if (strcmp(cmd, "socket") == 0)
+        {
+            acc->mask |= ACC_SOCKET;
+            free(acc->socketname);
+            if (*arg == '\0')
+            {
+                acc->socketname = NULL;
+            }
+            else
+            {
+                acc->socketname = xstrdup(arg);
+            }
+        }
         else if (strcmp(cmd, "tls_force_sslv3") == 0)
         {
             /* compatibility with versions <= 1.0.29: silently ignore */
