@@ -4,7 +4,7 @@
  * This file is part of mpop, a POP3 client.
  *
  * Copyright (C) 2000, 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010, 2011,
- * 2012, 2013, 2014, 2015, 2016, 2018, 2019
+ * 2012, 2013, 2014, 2015, 2016, 2018, 2019, 2020
  * Martin Lambers <marlam@marlam.de>
  * Dimitrios Apostolou <jimis@gmx.net> (UID handling)
  * Martin Stenberg <martin@gnutiken.se> (passwordeval support)
@@ -162,7 +162,7 @@ int mpop_serverinfo(account_t *acc, int debug, char **errmsg, char **errstr)
     session = pop3_session_new(acc->pipelining, "", "", debug ? stdout : NULL);
 
     /* connect */
-    if ((e = pop3_connect(session, acc->proxy_host, acc->proxy_port,
+    if ((e = pop3_connect(session, acc->socketname, acc->proxy_host, acc->proxy_port,
                     acc->host, acc->port, acc->source_ip, acc->timeout,
                     &server_canonical_name, &server_address, errstr))
             != NET_EOK)
@@ -184,7 +184,7 @@ int mpop_serverinfo(account_t *acc, int debug, char **errmsg, char **errstr)
                         acc->tls_sha1_fingerprint, acc->tls_md5_fingerprint,
                         acc->tls_min_dh_prime_bits,
                         acc->tls_priorities,
-                        acc->host,
+                        acc->tls_host_override ? acc->tls_host_override : acc->host,
                         acc->tls_nocertcheck,
                         errstr)) != TLS_EOK)
         {
@@ -661,7 +661,7 @@ int mpop_retrmail(const char *canonical_hostname, const char *local_user,
             debug ? stdout : NULL);
 
     /* connect */
-    if ((e = pop3_connect(session, acc->proxy_host, acc->proxy_port,
+    if ((e = pop3_connect(session, acc->socketname, acc->proxy_host, acc->proxy_port,
                     acc->host, acc->port, acc->source_ip, acc->timeout,
                     NULL, NULL, errstr)) != NET_EOK)
     {
@@ -680,7 +680,7 @@ int mpop_retrmail(const char *canonical_hostname, const char *local_user,
                         acc->tls_sha1_fingerprint, acc->tls_md5_fingerprint,
                         acc->tls_min_dh_prime_bits,
                         acc->tls_priorities,
-                        acc->host,
+                        acc->tls_host_override ? acc->tls_host_override : acc->host,
                         acc->tls_nocertcheck,
                         errstr)) != TLS_EOK)
         {
@@ -1425,36 +1425,38 @@ int make_needed_dirs(const char *pathname)
  */
 
 /* long options without a corresponding short option */
-#define LONGONLYOPT_VERSION                     0
-#define LONGONLYOPT_HELP                        1
-#define LONGONLYOPT_HOST                        2
-#define LONGONLYOPT_PORT                        3
-#define LONGONLYOPT_TIMEOUT                     4
-#define LONGONLYOPT_PIPELINING                  5
-#define LONGONLYOPT_RECEIVED_HEADER             6
-#define LONGONLYOPT_AUTH                        7
-#define LONGONLYOPT_USER                        8
-#define LONGONLYOPT_PASSWORDEVAL                9
-#define LONGONLYOPT_TLS                         10
-#define LONGONLYOPT_TLS_STARTTLS                11
-#define LONGONLYOPT_TLS_TRUST_FILE              12
-#define LONGONLYOPT_TLS_CRL_FILE                13
-#define LONGONLYOPT_TLS_FINGERPRINT             14
-#define LONGONLYOPT_TLS_KEY_FILE                15
-#define LONGONLYOPT_TLS_CERT_FILE               16
-#define LONGONLYOPT_TLS_CERTCHECK               17
-#define LONGONLYOPT_TLS_FORCE_SSLV3             18
-#define LONGONLYOPT_TLS_MIN_DH_PRIME_BITS       19
-#define LONGONLYOPT_TLS_PRIORITIES              20
-#define LONGONLYOPT_KILLSIZE                    21
-#define LONGONLYOPT_SKIPSIZE                    22
-#define LONGONLYOPT_FILTER                      23
-#define LONGONLYOPT_DELIVERY                    24
-#define LONGONLYOPT_UIDLS_FILE                  25
-#define LONGONLYOPT_PROXY_HOST                  26
-#define LONGONLYOPT_PROXY_PORT                  27
-#define LONGONLYOPT_SOURCE_IP                   28
-#define LONGONLYOPT_CONFIGURE                   29
+#define LONGONLYOPT_VERSION                     (256 + 0)
+#define LONGONLYOPT_HELP                        (256 + 1)
+#define LONGONLYOPT_HOST                        (256 + 2)
+#define LONGONLYOPT_PORT                        (256 + 3)
+#define LONGONLYOPT_TIMEOUT                     (256 + 4)
+#define LONGONLYOPT_PIPELINING                  (256 + 5)
+#define LONGONLYOPT_RECEIVED_HEADER             (256 + 6)
+#define LONGONLYOPT_AUTH                        (256 + 7)
+#define LONGONLYOPT_USER                        (256 + 8)
+#define LONGONLYOPT_PASSWORDEVAL                (256 + 9)
+#define LONGONLYOPT_TLS                         (256 + 10)
+#define LONGONLYOPT_TLS_STARTTLS                (256 + 11)
+#define LONGONLYOPT_TLS_TRUST_FILE              (256 + 12)
+#define LONGONLYOPT_TLS_CRL_FILE                (256 + 13)
+#define LONGONLYOPT_TLS_FINGERPRINT             (256 + 14)
+#define LONGONLYOPT_TLS_KEY_FILE                (256 + 15)
+#define LONGONLYOPT_TLS_CERT_FILE               (256 + 16)
+#define LONGONLYOPT_TLS_CERTCHECK               (256 + 17)
+#define LONGONLYOPT_TLS_FORCE_SSLV3             (256 + 18)
+#define LONGONLYOPT_TLS_MIN_DH_PRIME_BITS       (256 + 19)
+#define LONGONLYOPT_TLS_PRIORITIES              (256 + 20)
+#define LONGONLYOPT_TLS_HOST_OVERRIDE           (256 + 21)
+#define LONGONLYOPT_KILLSIZE                    (256 + 22)
+#define LONGONLYOPT_SKIPSIZE                    (256 + 23)
+#define LONGONLYOPT_FILTER                      (256 + 24)
+#define LONGONLYOPT_DELIVERY                    (256 + 25)
+#define LONGONLYOPT_UIDLS_FILE                  (256 + 26)
+#define LONGONLYOPT_PROXY_HOST                  (256 + 27)
+#define LONGONLYOPT_PROXY_PORT                  (256 + 28)
+#define LONGONLYOPT_SOURCE_IP                   (256 + 29)
+#define LONGONLYOPT_CONFIGURE                   (256 + 30)
+#define LONGONLYOPT_SOCKET                      (256 + 31)
 
 int main(int argc, char *argv[])
 {
@@ -1505,10 +1507,6 @@ int main(int argc, char *argv[])
     void (*old_sigterm_handler)(int);
     void (*old_sighup_handler)(int);
     void (*old_sigint_handler)(int);
-#endif
-    /* misc */
-#if HAVE_GETSERVBYNAME
-    struct servent *se;
 #endif
     int c;
     int net_lib_initialized = 0;
@@ -1552,6 +1550,7 @@ int main(int argc, char *argv[])
         { "proxy-host",            required_argument, 0, LONGONLYOPT_PROXY_HOST },
         { "proxy-port",            required_argument, 0, LONGONLYOPT_PROXY_PORT },
         { "source-ip",             required_argument, 0, LONGONLYOPT_SOURCE_IP },
+        { "socket",                required_argument, 0, LONGONLYOPT_SOCKET },
         { "tls",                   optional_argument, 0, LONGONLYOPT_TLS },
         { "tls-starttls",          optional_argument, 0,
             LONGONLYOPT_TLS_STARTTLS },
@@ -1573,6 +1572,8 @@ int main(int argc, char *argv[])
             LONGONLYOPT_TLS_MIN_DH_PRIME_BITS },
         { "tls-priorities",        required_argument, 0,
             LONGONLYOPT_TLS_PRIORITIES },
+        { "tls-host-override",     required_argument, 0,
+            LONGONLYOPT_TLS_HOST_OVERRIDE },
         { 0, 0, 0, 0 }
     };
 
@@ -1838,6 +1839,19 @@ int main(int argc, char *argv[])
                 cmdline_account->mask |= ACC_SOURCE_IP;
                 break;
 
+            case LONGONLYOPT_SOCKET:
+                free(cmdline_account->socketname);
+                if (*optarg)
+                {
+                    cmdline_account->socketname = xstrdup(optarg);
+                }
+                else
+                {
+                    cmdline_account->socketname = NULL;
+                }
+                cmdline_account->mask |= ACC_SOCKET;
+                break;
+
             case LONGONLYOPT_TLS:
                 if (!optarg || is_on(optarg))
                 {
@@ -1992,6 +2006,19 @@ int main(int argc, char *argv[])
                 cmdline_account->mask |= ACC_TLS_PRIORITIES;
                 break;
 
+            case LONGONLYOPT_TLS_HOST_OVERRIDE:
+                free(cmdline_account->tls_host_override);
+                if (*optarg)
+                {
+                    cmdline_account->tls_host_override = xstrdup(optarg);
+                }
+                else
+                {
+                    cmdline_account->tls_host_override = NULL;
+                }
+                cmdline_account->mask |= ACC_TLS_HOST_OVERRIDE;
+                break;
+
             case 'n':
                 if (!optarg || is_on(optarg))
                 {
@@ -2229,7 +2256,7 @@ int main(int argc, char *argv[])
             free(conffile);
         }
         printf("\n");
-        printf(_("Copyright (C) 2019 Martin Lambers and others.\n"
+        printf(_("Copyright (C) 2020 Martin Lambers and others.\n"
                     "This is free software.  You may redistribute copies of "
                         "it under the terms of\n"
                     "the GNU General Public License "
@@ -2264,9 +2291,10 @@ int main(int argc, char *argv[])
         printf(_("  --host=hostname              set the server, use only command-line settings;\n"
                     "                               do not use any configuration file data\n"));
         printf(_("  --port=number                set port number\n"));
+        printf(_("  --source-ip=[IP]             set/unset source ip address to bind the socket to\n"));
         printf(_("  --proxy-host=[IP|hostname]   set/unset proxy\n"));
         printf(_("  --proxy-port=[number]        set/unset proxy port\n"));
-        printf(_("  --source-ip=[IP]             set/unset source ip address to bind the socket to\n"));
+        printf(_("  --socket=[socketname]        set/unset local socket to connect to\n"));
         printf(_("  --timeout=(off|seconds)      set/unset network timeout in seconds\n"));
         printf(_("  --pipelining=(auto|on|off)   enable/disable pipelining\n"));
         printf(_("  --received-header[=(on|off)] enable/disable Received-header\n"));
@@ -2282,6 +2310,7 @@ int main(int argc, char *argv[])
         printf(_("  --tls-key-file=[file]        set/unset private key file for TLS\n"));
         printf(_("  --tls-cert-file=[file]       set/unset private cert file for TLS\n"));
         printf(_("  --tls-priorities=[prios]     set/unset TLS priorities.\n"));
+        printf(_("  --tls-host-override=[host]   set/unset override for TLS host verification.\n"));
         printf(_("  --tls-min-dh-prime-bits=[b]  set/unset minimum bit size of DH prime\n"));
         printf(_("Options specific to mail retrieval mode:\n"));
         printf(_("  -q, --quiet                  do not display status or progress information\n"));
@@ -2472,6 +2501,8 @@ int main(int argc, char *argv[])
             printf("proxy host = %s\n",
                     account->proxy_host ? account->proxy_host : _("(not set)"));
             printf("proxy port = %d\n", account->proxy_port);
+            printf("socket = %s\n",
+                    account->socketname ? account->socketname : _("(not set)"));
             printf("timeout = ");
             if (account->timeout <= 0)
             {
@@ -2551,6 +2582,8 @@ int main(int argc, char *argv[])
             }
             printf("tls_priorities = %s\n", account->tls_priorities
                     ? account->tls_priorities : _("(not set)"));
+            printf("tls_host_override = %s\n", account->tls_host_override
+                    ? account->tls_host_override : _("(not set)"));
             if (retrmail)
             {
                 printf("delivery = ");
@@ -2665,7 +2698,7 @@ int main(int argc, char *argv[])
         {
             if (!canonical_hostname)
             {
-                canonical_hostname = net_get_canonical_hostname();
+                canonical_hostname = net_get_canonical_hostname(NULL);
             }
             if (!local_user)
             {
