@@ -4,7 +4,7 @@
  * This file is part of mpop, a POP3 client.
  *
  * Copyright (C) 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010, 2014, 2015,
- * 2016, 2018, 2019
+ * 2016, 2018, 2019, 2020
  * Martin Lambers <marlam@marlam.de>
  *
  *   This program is free software; you can redistribute it and/or modify
@@ -228,7 +228,7 @@ void pop3_session_free(pop3_session_t *session);
 /*
  * pop3_connect()
  *
- * Connect to a POP3 server.
+ * Connect to a POP3 server. See net_open_socket().
  * If 'server_canonical_name' is not NULL, a pointer to a string containing the
  * canonical hostname of the server will be stored in '*server_canonical_name',
  * or NULL if this information is not available.
@@ -239,6 +239,7 @@ void pop3_session_free(pop3_session_t *session);
  * Used error codes: NET_EHOSTNOTFOUND, NET_ESOCKET, NET_ECONNECT, NET_EPROXY
  */
 int pop3_connect(pop3_session_t *session,
+        const char *socketname,
         const char *proxy_hostname, int proxy_port,
         const char *server_hostname, int port, const char *source_ip, int timeout,
         const char **server_canonical_name, const char **server_address,
@@ -346,7 +347,7 @@ int pop3_server_supports_authmech(pop3_session_t *session, const char *mech);
  * to find out which authentication mechanisms are available.
  * The special value "" for 'auth_mech' causes the function to choose the best
  * authentication method supported by the server, unless TLS is inactive and the
- * method sends plain text passwords. In this case, the function fails with
+ * method sends cleartext passwords. In this case, the function fails with
  * POP3_EINSECURE.
  * The hostname is the name of the POP3 server. It may be needed for
  * authentication.
